"use strict";

this.name        = "ups_parcel";
this.author      = "eric walch";
this.copyright   = "2008 eric walch.";
this.license     = "CC BY-NC-SA 3.0";
this.description = "Parcel missions of UPS Courier";

this.startUp = function ()
{
    this.pirate = "pirate";
    this.tiger =  (worldScripts.TigersTurf) ? "tigers" : this.pirate;
    this.dictatorsInstalled = (worldScripts.dictatorship_population !== undefined || worldScripts["dictators.js"]) ? true : false; // test for old legacy script and 1.4+ js script
    this.mark = false;
    this.upsMarkSystem = worldScripts["ups_docs"].upsMarkSystem;
    this.upsUnmarkSystem = worldScripts["ups_docs"].upsUnmarkSystem;
}

this.shipWillDockWithStation = function (station)
{
    if (station.isMainStation)
    {
        this.ups_reports(station);
    }
}

this.missionScreenOpportunity = function ()
{
    if (player.ship.docked)
    {
        this.missionOffers();
    }
}

this.ups_reports = function (station)
{
    if (this.ups_parcel === "SMALL_DELIVERY" && system.government === 4 &&
                this.ups_pplanet !==  system.ID && this.ups_pcount > 0 &&
                missionVariables.ups_ambush != "YES")
    {
        this.reportDelivery(150);
    }
    if (this.ups_parcel === "MIDAIR_DELIVERY" && this.ups_pplanet ===  system.ID)
    {
        this.reportDelivery(150);
    }
}

this.addUpsContract = function ()
{
    var thisSystem = system.info;

    var targetSystems = SystemInfo.filteredSystems(this, function(other){
           return (other.systemID !== thisSystem.systemID) && other.government == 4 && !system.sun.hasGoneNova;
    });
    var count = Math.ceil(Math.random()*5);

    for (var i = 0; i<count; i++)
    {
        var contractSystem = targetSystems[Math.floor(Math.random() * targetSystems.length)];
        var contractRoute = system.info.routeToSystem(contractSystem);
        if (contractRoute) {
			var contractDescription = expandDescription("[ups_colors] [ups_parcel_description]");
			contractDescription = contractDescription.charAt(0).toUpperCase() + contractDescription.slice(1);

			var parcel =
			{
				destination: contractSystem.systemID,
				sender: "UPS courier",
				deadline: clock.seconds + contractRoute.time * 3600 + 3600*24*2,
				payment: Math.floor(contractRoute.distance * (3+Math.random()) + (Math.pow(contractRoute.route.length,1+(0.3*player.parcelReputation))) + 50),
				description: contractDescription,
				route: contractRoute
			}

	        worldScripts["oolite-contracts-parcels"]._addParcelToSystem(parcel);
		}
    }
};

this.reportDelivery = function (payment)
{
    var message = expandDescription("You deliver the [mission_ups_parcel_appearance] parcels to the UPS office at the station. In return you receive "+payment+" ₢.");
    player.addMessageToArrivalReport(message);
    player.credits += payment;
    this.ups_pcount++;
    mission.setInstructionsKey(null);
    this.ups_parcel = "NOT_NOW";
    this.upsUnmarkSystem(this.ups_pplanet, this.name);
    missionVariables.ups_pplanetname = null;
    missionVariables.ups_ambush = null;
    missionVariables.ups_parcel_appearance = null;
    worldScripts["ups_docs"].increaseReputation(1);
}

this.guiScreenChanged = function (newScreen, oldScreen)
{
    if (this.restartMissionOffer && newScreen !== "GUI_SCREEN_LONG_RANGE_CHART")
    {
        this.restartMissionOffer = false;
        if (guiScreen !== "GUI_SCREEN_MISSION") this.missionOffers();
    }
}

this.missionOffers = function ()
{
    if (this._savedTargetSet) player.ship.targetSystem = this._savedTarget
    this._savedTarget = null;
    this._savedTargetSet = false;

	var parcelStatus = this.ups_parcel // for speed reasons don't test to often against a mission variable.
    if (player.ship.dockedStation.isMainStation)
    {
        if (parcelStatus === "YES")
        {
            if (system.government === 4 && this.ups_pcount > 20 && player.ship.bounty < 3 &&
                        this._reputation() > 4 && (Math.random() < 0.25 || this.mark) && !worldScripts["ups_docs"].lookLongRangeChartScreen)
            {
                // Medicine offer.
                if (!this.mark)
                {
                    // a nova system has the market set. Also avoid other systems with customised markets from selection.
                    this.ups_pplanet = Math.round(Math.random()*255);
                }

				missionVariables.ups_pplanetname = System.systemNameForID(this.ups_pplanet);
				missionVariables.ups_parcel_appearance = expandDescription("[ups_numbers] [ups_colors] %I");
				if (!this.mark) missionVariables.ups_disease = expandDescription("[ups_diseases]");
                missionVariables.ups_medicine_inhabitants = System.infoForSystem(galaxyNumber, this.ups_pplanet).inhabitants;
                var message = expandMissionText("ups_parcel_medicin_offer", {dockedStationName: player.ship.dockedStation.displayName});
				mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", message: message,
                                overlay: this._overlay(), choicesKey: "ups_parcel_accepted_yesnolook"},
                                this.choiceEvaluation);
                this.offering ="UPS_PARCEL_MEDICINS";
                missionVariables.ups_medicine_inhabitants = null;
            }
            else if (system.government === 4 && worldScripts["ups_docs"].ups_dcount > 2 && player.ship.bounty < 3 && this._reputation() > 2)
            {
                // Parcel offer
                missionVariables.ups_parcel_appearance = expandDescription("[ups_numbers] [ups_colors] %I")
                var today = new Date()
                if (today.getMonth()==11 && today.getDate() > 15 && today.getDate() < 26)
                    missionVariables.ups_parcel_appearance += " Christmas";
                if (!this.ups_pcount)
                    mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_1st_offer",
                                        overlay: this._overlay(), choicesKey: "ups_parcel_accepted_yesno"},
                                        this.choiceEvaluation)
                else mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_offer",
                                        overlay: this._overlay(), choicesKey: "ups_parcel_accepted_yesno"},
                                        this.choiceEvaluation);
                this.offering ="UPS_PARCELS";
            }
            else this.setUpsReputation();
        }
        if (parcelStatus === "SMALL_DELIVERY")
        {
            if (system.government === 4)
            {
                if (this.ups_pplanet === system.ID)
                {
                    this.ups_parcel = "ACCEPTED";
                    mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_back_at_home_parcels", overlay: this._overlay()});
                    worldScripts["ups_docs"].increaseReputation(-1);
                }
                else
                {
                    this.ups_parcel = "NOT_NOW";
                    player.credits += 150;
                    mission.setInstructionsKey(null);
                    if (!this.ups_pcount)
                    {
                        mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_1st_unloading", overlay: this._overlay()});
                    }
                    else
                    {
                        if (missionVariables.ups_ambush === "YES")
                        {
                            mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_ambush", overlay: this._overlay()});
                            player.credits += 500;
                        }
                        // else // written in arrival report
                    }
                    this.ups_pcount++;
                    worldScripts["ups_docs"].increaseReputation(1);
                    missionVariables.ups_parcel_appearance = null;
                    missionVariables.ups_ambush = null;
                    missionVariables.ups_pplanetname = null;
                    if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_parcels]")});
                    if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_parcels]")});
                }
            }
        }
        if (parcelStatus === "MEDICIN_DELIVERY")
        {
            var passedTime = clock.minutes/24/60 - this.ups_ptimerstart; // time in days.
            if (this.ups_pplanet === system.ID)
            {
                missionVariables.ups_medicine_government = system.government;
                missionVariables.ups_medicine_population = system.population;
                missionVariables.ups_medicine_productivity = system.productivity;
                missionVariables.ups_medicine_description = system.description;
                missionVariables.ups_medicine_inhabitants = system.inhabitantsDescription;
                mission.setInstructionsKey(null);
                this.upsUnmarkSystem(this.ups_pplanet, this.name);
                this.ups_ptimerstart = clock.days;

                if (passedTime < 15)
                {
                    this.ups_parcel = "MEDICIN_SUCCESS";
                    mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", messageKey: "ups_parcel_medicin_unloading", overlay: this._overlay()});
                    player.credits += 4500;
                    this.ups_pcount++;
                    worldScripts["ups_docs"].increaseReputation(4);
                }
                else
                {
                    this.ups_parcel = "MEDICIN_FAILURE";
                    mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", messageKey: "ups_parcel_medicin_unloading_late", overlay: this._overlay()});
                    worldScripts["ups_docs"].increaseReputation(-3);
                    this.ups_ptimerstart = clock.days;
                }
            }
            else if (passedTime > 30)
            {
                this.ups_parcel = "NOT_NOW";
                mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", messageKey: "ups_parcel_medicin_expired", overlay: this._overlay()});
                mission.setInstructionsKey(null);
                this.resetMedicines(galaxyNumber, false);
                this.upsUnmarkSystem(this.ups_pplanet);
                worldScripts["ups_docs"].increaseReputation(-6);
            }
            else
            {
                this.updateMessages();
            }
        }
        if (parcelStatus === "MEDICIN_SUCCESS")
        {
            if (this.ups_pplanet != system.ID  && (clock.hours/24 - this.ups_ptimerstart) > 10)
            {
                missionVariables.ups_parcel_appearance = expandDescription("[ups_numbers] [ups_colors] %I");
                var message = expandMissionText("ups_parcel_medicin_offer_2a", {dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", message: message,
                                    overlay: this._overlay(), choicesKey: "ups_parcel_accepted_yesno"},
                                    this.choiceEvaluation);
                this.offering ="UPS_PARCEL_MEDICINS_2";
            }
            else this.setUpsReputation();
        }
        if (parcelStatus === "MEDICIN_FAILURE")
        {
            if (this.ups_pplanet != system.ID  && (clock.hours/24 - this.ups_ptimerstart) > 10)
            {
                missionVariables.ups_parcel_appearance = expandDescription("[ups_numbers] [ups_colors] %I");
                var message = expandMissionText("ups_parcel_medicin_offer_2b", {dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", message: message,
                                    overlay: this._overlay(), choicesKey: "ups_parcel_accepted_yesno"},
                                    this.choiceEvaluation);
                this.offering ="UPS_PARCEL_MEDICINS_2";
            }
            else this.setUpsReputation();
        }
        if (parcelStatus === "MEDICIN_DELIVERY_2" && this.ups_pplanet === system.ID)
        {
            this.ups_parcel = "NOT_NOW";
            var success = false;
            if ((clock.hours/24 - this.ups_ptimerstart) < 15)
            {
                mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", messageKey: "ups_parcel_medicin_unloading_2", overlay: this._overlay()});
                player.credits += 4500;
                this.ups_pcount++;
                worldScripts["ups_docs"].increaseReputation(4);
                success = true;
            }
            else
            {
                mission.runScreen({screenID: "ups-parcel", title: "Medicine Transport", messageKey: "ups_parcel_medicin_unloading_late", overlay: this._overlay()});
                worldScripts["ups_docs"].increaseReputation(-5);
            }

            this.resetMedicines(galaxyNumber, success);
            this.upsUnmarkSystem(this.ups_pplanet, this.name);
            missionVariables.ups_pplanetname = null;
            missionVariables.ups_museumGoods = null;
            mission.setInstructionsKey(null);
        }
        if (parcelStatus === "LOGGING")
        {
            if (system.government === 4 && player.credits > 15000 && this.ups_pplanet != system.ID && (clock.days - this.ups_ptimerstart) > 8 && this._reputation() > 3 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
            {
                if (!this.mark) //has not seen this offer before, aka not used the long range chart option.
                {
                    missionVariables.ups_museumGoods = expandDescription("[ups_museumGoods]");
                    if (!missionVariables.ups_trumbles && missionVariables.trumbles === "TRUMBLE_BOUGHT" && Math.random() < 0.75 && player.trumbleCount === 0)
                    {
                        if (player.ship.equipmentStatus("EQ_TRUMBLE") === "EQUIPMENT_OK") player.ship.removeEquipment("EQ_TRUMBLE");
                        missionVariables.ups_museumGoods = expandDescription("[ups_trumbles]");
                        missionVariables.ups_trumbles = "ESCAPED";
                    }
                }
                var message = expandMissionText("ups_parcel_offer2", {dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", message: message,
                                    overlay: this._overlay(), choicesKey: "ups_parcel_museum_accepted_yesnolook"},
                                    this.choiceEvaluation);
                this.offering = "UPS_PARCELS2";
            }
            else this.setUpsReputation();
        }
        if (parcelStatus === "SPECIAL_DELIVERY")
        {
            if (this.ups_pplanet == system.ID)
            {
                this.ups_parcel = "NOT_NOW";
                if (missionVariables.ups_trumbles === "ESCAPED")
                {
                    mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_trumbles", overlay: "trumblebox.png"});
                    player.credits -= 1000;
                    missionVariables.ups_trumbles = "YES";
					this.playSound("trumblesqueal.ogg");
                }
                else
                {
                    mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_parcel_unloading2", overlay: this._overlay()})
                }
                player.credits += 15000;
                this.ups_pcount++;
                mission.setInstructionsKey(null);
                missionVariables.ups_pplanetname = null;
                missionVariables.ups_museumGoods = null;
                missionVariables.ups_escorts = null;
                this.upsUnmarkSystem(this.ups_pplanet, this.name);
                this.ups_ptimerstart = null;
                worldScripts["ups_docs"].increaseReputation(3);
				 // clean up escorts by docking them.
				var escorts = system.shipsWithPrimaryRole("escort_ups", player.ship);
				for (var i=0; i< escorts.length;i++) if (escorts[i].isValid) escorts[i].switchAI("dockingAI.plist");
             }
        }
        if (parcelStatus === "ASTRO_OFFER" && system.government === 3 && system.economy < 3)
        {
            if (system.countShipsWithRole("astrofactory") > 0)
            {
                missionVariables.ups_parcel_appearance = expandDescription("[ups_numbers] astro factory");
                var message = expandMissionText("ups_parcel_astroAttack", {dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", message: message, model: "astrofactory",
                                    overlay: this._overlay(), choicesKey: "ups_astro_parcel_accepted_yesno"},
                                    this.choiceEvaluation);
                this.addAstroPirates(20);
                this.offering = "UPS_PARCEL_ASTRO";
            }
            else parcelStatus = this.ups_parcel = "NO";
        }
        if (parcelStatus === "ASTRO_ACCEPTED" && system.countShipsWithRole("astrofactory") === 0)
        {
            this.ups_parcel = "NOT_NOW";
            if (this.ups_pplanet === system.ID)
                            mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_astro_parcel_destroyed", overlay: this._overlay()});
            mission.setInstructionsKey(null);
            this.upsUnmarkSystem(this.ups_pplanet, this.name);
            missionVariables.ups_pplanetname = null;
            missionVariables.ups_parcel_appearance = null;
            worldScripts["ups_docs"].increaseReputation(-2);
        }
        if (parcelStatus === "ASTRO_DELIVERY" && this.ups_pplanet === system.ID)
        {
            this.ups_parcel = "NOT_NOW";
            mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_astro_parcel_unloading", overlay: this._overlay()});
            player.credits += 250;
            this.ups_pcount++;
            mission.setInstructionsKey(null);
            this.upsUnmarkSystem(this.ups_pplanet, this.name);
            missionVariables.ups_pplanetname = null;
            missionVariables.ups_parcel_appearance = null;
            worldScripts["ups_docs"].increaseReputation(3);
        }
		if (parcelStatus === "NO" || this.ups_parcel === "NOT_NOW")
		{
            this.setUpsReputation();
		}
    };
	// Not at a mainStation
    if (parcelStatus === "ASTRO_ACCEPTED" && player.ship.dockedStation.name === "Imperial AstroFactory")
    {
        this.ups_parcel = "ASTRO_DELIVERY";
        mission.runScreen({screenID: "ups-parcel", title: "Parcel Transport", messageKey: "ups_astro_parcel_loading", overlay: this._overlay()});
        mission.setInstructionsKey("ups_astro_parcel_small2");
        var astroPirates = system.shipsWithPrimaryRole("pirate", player.ship, 40E3); // all pirates in a radius of 40000 m around the player
        for (var i=0; i<astroPirates.length;i++)
        {
            if (astroPirates[i].isValid)
            {
                astroPirates[i].switchAI("upsAmbushPirateAI.plist");
                if (astroPirates[i].script.name != "oolite-default-ship-script") astroPirates[i].setScript("oolite-default-ship-script.js");
            }
        }
        player.ship.fuel = 7;
    }
}

this.choiceEvaluation = function (choice)
{
    switch (this.offering)
    {
        case "UPS_PARCELS":
        {
            if (choice === "YESParcel")
            {
                mission.setInstructionsKey("ups_parcel_small");
                this.ups_parcel = "ACCEPTED";
                this.ups_pplanet = system.ID;
                missionVariables.ups_pplanetname = system.name;
                if (Math.random() < 0.2 && this.ups_pcount > 1) missionVariables.ups_ambush = "POSSIBLE"
            }
            else if (choice === "NOParcel")
            {
                this.ups_parcel = "NOT_NOW";
                missionVariables.ups_parcel_appearance = null;
                worldScripts["ups_docs"].increaseReputation(-2);
            }
            break;
        }
        case "UPS_PARCELS2":
        {
            this.mark = false;
            if (choice === "YESParcel")
            {
                mission.setInstructionsKey("ups_parcel_small2", "ups_parcel");
                this.ups_parcel = "SPECIAL_DELIVERY";
                this.upsMarkSystem(this.ups_pplanet, this.name);
                player.credits -= 10000;
                missionVariables.ups_escorts = 8;
            }
            else if (choice === "LOOKParcel")
            {
                this.upsShowDestination(this.ups_pplanet);
            }
            else if (choice === "NOParcel")
            {
                this.ups_parcel = "NOT_NOW";
                worldScripts["ups_docs"].increaseReputation(-2);
            }
            break;
        }
        case "UPS_PARCEL_MEDICINS":
        {
            this.mark = false;
            if (choice === "YESParcel")
            {
                mission.setInstructions(expandMissionText("ups_parcel_small3", {ups_days_left: "within 15 days"}));
                this.ups_parcel = "MEDICIN_DELIVERY";
                this.upsMarkSystem(this.ups_pplanet, this.name);
                this.ups_ptimerstart = clock.minutes/24/60; // time in days
                missionVariables.ups_museumGoods = missionVariables.ups_disease + " medicine";
            }
            else if (choice === "LOOKParcel")
            {
                this.upsShowDestination(this.ups_pplanet);
            }
            else if (choice === "NOParcel")
            {
                this.ups_parcel = "NOT_NOW";
                worldScripts["ups_docs"].increaseReputation(-2);
            }
            break;
        }
        case "UPS_PARCEL_MEDICINS_2":
        {
            if (choice === "YESParcel")
            {
                mission.setInstructions(expandMissionText("ups_parcel_small3", {ups_days_left: "within 15 days"}));
                this.ups_parcel = "MEDICIN_DELIVERY_2";
                this.upsMarkSystem(this.ups_pplanet, this.name);
                this.ups_ptimerstart = clock.minutes/24/60;
                var targetSystem = System.infoForSystem(galaxyNumber, this.ups_pplanet);
                targetSystem.description = "The "+ missionVariables.ups_medicine_inhabitants+ " that are living on this planet are dying from an outbreak of " + missionVariables.ups_disease+".";
                if (missionVariables.ups_medicine_inhabitants.indexOf("Very pale looking") < 0) // don't change this twice.
                    targetSystem.inhabitants = "Very pale looking " + missionVariables.ups_medicine_inhabitants;
                targetSystem.population = Math.round(missionVariables.ups_medicine_population/10);
                targetSystem.productivity = Math.round(missionVariables.ups_medicine_productivity/40);
                targetSystem.government = 0;
            }
            else if (choice === "NOParcel")
            {
                this.ups_parcel = "NOT_NOW";
                worldScripts["ups_docs"].increaseReputation(-2);
                this.resetMedicines(galaxyNumber, false);
            }
            break;
        }
        case "UPS_PARCEL_ASTRO":
        {
            if (choice === "YESParcel")
            {
                mission.setInstructionsKey("ups_astro_parcel_small");
                this.ups_parcel = "ASTRO_ACCEPTED";
                missionVariables.ups_parcel_appearance = "imperial astrofactory";
                this.ups_pplanet = system.ID;
                missionVariables.ups_pplanetname = system.name;
                this.upsMarkSystem(this.ups_pplanet, this.name);
            }
            else if (choice === "NOParcel")
            {
                this.ups_parcel = "NOT_NOW";
                worldScripts["ups_docs"].increaseReputation(-2);
            }
            break;
        }
        default: log(this.name, "Encountered an illegal mission offer: "+this.offering);
    }
    this.offering = null;
}

this.upsShowDestination = function (ID)
{
    if (!this._savedTargetSet) this._savedTarget = player.ship.targetSystem;
    this._savedTargetSet = true;
    
    this.mark = true; // Keep this selected system.
    if (player.ship.equipmentStatus("EQ_ADVANCED_NAVIGATIONAL_ARRAY") == "EQUIPMENT_OK")
    {
        player.ship.targetSystem = ID;
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", backgroundSpecial: "LONG_RANGE_CHART_SHORTEST"});
    }
    else
    {
        mission.markSystem({
                   system: ID,
                   name: "ups_marker",
                   markerColor: "cyanColor",
                   markerScale: 2.0,
                   markerShape: "MARKER_DIAMOND"
        })
        var message = expandMissionText("ups_long_range_screen", {ups_system: System.systemNameForID(ID)});
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", message: message, backgroundSpecial: "LONG_RANGE_CHART"});
    }
}

this.addAstroPirates = function (amount)
{
    var astroFactory = system.shipsWithPrimaryRole("astrofactory")[0];
    var astroPirates = system.addGroup("pirate", amount, astroFactory.position.add([0,20E3,0]), 5E3).ships;
    for (var i=0; i<amount;i++)
        if (astroPirates[i].isValid)
        {
            astroPirates[i].switchAI("upsAstroPirateAI.plist");
            if (astroPirates[i].script.name != "oolite-default-ship-script") astroPirates[i].setScript("oolite-default-ship-script.js");
            if (astroPirates[i].bounty === 0) astroPirates[i].bounty = Math.random()*100;
        }
}

this.shipExitedWitchspace = function ()
{
    if (system.isInterstellarSpace) return;
	var parcelStatus = this.ups_parcel;
    if (parcelStatus === "NOT_NOW" && Math.random() < 0.11) this.ups_parcel = "NO";
    if (parcelStatus === "NO")
    {
        if (Math.random() < 0.2 && system.government === 4)
        {this.ups_parcel = "YES"}
        else if (Math.random() < 0.05 && system.government === 4 && player.credits > 15000 && !system.sun.hasGoneNova && this.ups_pcount > 5)
        {
            this.ups_parcel = "LOGGING";
            this.ups_pplanet = system.ID;
            missionVariables.ups_pplanetname = system.name;
            this.ups_ptimerstart = clock.days;
        }
        else if (Math.random() < 0.1)
        {
            if (system.government > 3 && system.economy > 3 && missionVariables.ups_tcount > 1 && this._reputation() > 3)
            system.addShipsToRoute("derelict2_ptt", 1, Math.random(), "wp");
        }
        else if (missionVariables.ups_trumbles === "YES")
        {
            if (Math.random() < 0.85) this.playTrumble = new Timer(this, this.trumbleSound, 30+ Math.random()*120);
			else
			{
				player.ship.awardEquipment("EQ_TRUMBLE");
				missionVariables.ups_trumbles = "INFECTED";
                // Variable stays "INFECTED" to prefent a second infection in this galaxy.
			}
        }
        else if (Math.random() < 0.2 && this.dictatorsInstalled && system.government === 3 && system.economy < 3 && this.ups_pcount > 10)
        {
            this.ups_parcel = "ASTRO_OFFER";
        }
    }
    else
    {
        if (parcelStatus === "ACCEPTED"){parcelStatus = "SMALL_DELIVERY"; this.ups_parcel = parcelStatus; }
        if (parcelStatus === "SMALL_DELIVERY" && !system.isInterstellarSpace)
        {
            system.addShipsToRoute(this.pirate, 3, 0.33)
            if (Math.random() < 0.1 && missionVariables.ups_ambush === "POSSIBLE")
            {
				missionVariables.ups_ambush = "YES";
				player.consoleMessage(expandDescription("You are spotted with your [mission_ups_parcel_appearance] parcels."), 6)
			}
			if (missionVariables.ups_ambush === "YES" && Math.random() < 0.5)
			{
				system.addShips(this.tiger, 1);
				system.addShipsToRoute(this.tiger, 2, 0.2, "wp");
				if (player.score > 1000 && (Math.random()*100 + this.ups_pcount - 2*system.government) > 70)
				{
					system.addGroupToRoute(this.pirate, 3, 0, "wp");
					system.addGroup(this.tiger, 3, this.randomPositionOnRoute(), 5E3);
					system.addShipsToRoute("ups_pirate_leader", 1, Math.random(), "wp");
				}
				if (player.score > 2500 && (Math.random()*100 + this.ups_pcount - 2*system.government) > 70)
				{
					system.addGroupToRoute(this.tiger, 3, 0, "wp");
					system.addGroup(this.tiger, 3, this.randomPositionOnRoute(), 5E3);
					system.addShipsToRoute("ups_pirate_leader", 1, Math.random(), "wp");
					if (worldScripts.hardpirates1 !== undefined)
					{
						var hardpirates = system.addShips("hardpirates", 1);
                        this.changePirateAI(hardpirates[0], "upsAmbushPirateAI.plist", this.sendUpsAmbushMessage);
					}
				}
				var ambushpirates = system.shipsWithPrimaryRole("pirate", player.ship);
				for (var i=0; i<ambushpirates.length;i++)
                {
					if (ambushpirates[i].isValid) //could have already been destroyed or not present
					{
						this.changePirateAI(ambushpirates[i], "upsAmbushPirateAI.plist", this.sendUpsAmbushMessage);
                        if (player.score > 2500 * Math.random()) this.awardShields(ambushpirates[i], "EQ_SHIELD_BOOSTER");
                        if (player.score > 4500 * Math.random()) this.awardShields(ambushpirates[i], "EQ_SHIELD_ENHANCER");
					}
                }
			}
        }
        if (parcelStatus === "SPECIAL_DELIVERY")
        {
            if (system.countShipsWithPrimaryRole("escort_ups") == 0 && missionVariables.ups_escorts > 0)
                                        system.addShips("escort_ups", missionVariables.ups_escorts, player.ship.position, 10000);
            if (Math.random() < 0.5)
            {
                system.addGroupToRoute(this.pirate, 4, 0.05, "wp");
                system.addGroupToRoute(this.pirate, 3, 0.075, "wp");
                system.addGroupToRoute(this.pirate, 3, 0.10, "wp");
                system.addGroupToRoute(this.pirate, 2, 0.125, "wp");
                system.addShipsToRoute(this.pirate, 1, 0.15, "wp");
                var parcelpirates1 = system.shipsWithPrimaryRole(this.pirate, player.ship);
                for (var i=0; i<13;i++) this.changePirateAI(parcelpirates1[i], "upsParcelPirateAI.plist", this.sendUpsParcelMessage);
            }
            if (Math.random() < 0.25)
            {
				var parcelpirates2 = system.addGroupToRoute(this.tiger, 5, 0, "wp");
				for (var i=0; i<5;i++) this.changePirateAI(parcelpirates2.ships[i], "upsParcelPirateAI.plist", this.sendUpsParcelMessage);
            }
			this.addParcelPirates("pirate", 5);
			if (missionVariables.ups_trumbles && missionVariables.ups_trumbles !== "INFECTED" && Math.random() < 0.50)
            {
                this.playTrumble = new Timer(this, this.trumbleSound, 30+ Math.random()*120);
            }
        }
        if (parcelStatus === "MEDICIN_DELIVERY")
		{
			if (Math.random() < 0.25) this.addParcelPirates("pirate", 5);
            this.updateMessages();
		}
        if (parcelStatus === "MEDICIN_DELIVERY_2")
		{
			if (Math.random() < 0.25) system.addGroup(this.tiger, 3, this.randomPositionOnRoute(), 5E3);
			if (Math.random() < 0.25) system.addGroup(this.tiger, 3, this.randomPositionOnRoute(), 5E3);
			if (Math.random() < 0.5) this.addParcelPirates("pirate", Math.ceil(Math.random()*5));

            this.updateMessages();
		}
		if (parcelStatus === "ASTRO_ACCEPTED")
		{
			mission.setInstructionsKey(null);
			this.ups_parcel = "NOT_NOW";
			this.upsUnmarkSystem(this.ups_pplanet);
			missionVariables.ups_pplanetname = null;
			missionVariables.ups_parcel_appearance = null;
		}
    }
    if (system.government === 4 && !system.sun.hasGoneNova && system.mainStation)
    {
        this.addUpsContract();
    }
}

this.updateMessages = function ()
{
    if (this.messagesTimer)
    {
        this.messagesTimer.start();
    }
    else
    {
        this.messagesTimer = new Timer(this, this.delayedMessageUpdate, 0, 2); // clock might be not valid yet.
    }
}

this.delayedMessageUpdate = function delayedMessageUpdate()
{
    // we need a delayed clock readout after a jump.
    if (clock.isAdjusting)
    {
        mission.setInstructions(expandMissionText("ups_parcel_small3", {ups_days_left: "updating...."}));
    }
    else
    {
        mission.setInstructions(expandMissionText("ups_parcel_small3",
                    {ups_days_left: worldScripts["ups_docs"].getTimeToGo(this.ups_ptimerstart + 15)}));
        this.messagesTimer.stop();
    }
}

this.addParcelPirates = function (role, count)
{
    var parcelpirates = system.addGroup(role, count, this.randomPositionOnRoute(), 5E3);
    if (parcelpirates)
    {
        parcelpirates.name = "parcelpirates";
        for (var i=0; i<parcelpirates.count; i++)
        {
            this.changePirateAI(parcelpirates.ships[i], "upsParcelPirateAI.plist", this.sendUpsParcelMessage);
            if (parcelpirates.ships[i].isValid)
            {
                this.awardShields(parcelpirates.ships[i], "EQ_SHIELD_ENHANCER");
            }
        }
    }
}

this.changePirateAI = function (pirate, AI, message)
{
    if (pirate && pirate.isValid)
    {
        pirate.switchAI(AI);
        // if (pirate.script.name != "oolite-default-ship-script") pirate.setScript("oolite-default-ship-script.js"); // should we reset this?
        pirate.script.sendUpsMessage = message; // message is the function to attach that sends the messages.
    }
}

this.randomPositionOnRoute = function ()
{
    var end = system.mainPlanet.position;
    end.z -= 2 * system.mainPlanet.radius;
    return Vector3D.interpolate([0,0,0], end, Math.random()).add(Vector3D.randomDirectionAndLength(20E3));
};

this.awardShields = function (ship, equipment)
{
    if (ship.equipmentStatus(equipment) === "EQUIPMENT_UNAVAILABLE")
    {
        ship.awardEquipment(equipment);
        ship.energy = ship.maxEnergy;
    }
}

this.playerEnteredNewGalaxy = function (galaxy)
{
    // reset missions on galactic jump
	if (--galaxy < 0) galaxy = 8;
	if (this.ups_parcel.indexOf("MEDICINE") > -1) this.resetMedicines(galaxy, false);
    if (this.ups_parcel != "SMALL_DELIVERY")
    {
            missionVariables.ups_pplanetname = null;
			missionVariables.ups_parcel_appearance = null;
            mission.setInstructionsKey(null);
            this.ups_pplanet = null;
            this.ups_parcel = "NOT_NOW";
            this.ups_ptimerstart = null;
    }
    if (player.ship.equipmentStatus("EQ_TRUMBLE") !== "EQUIPMENT_OK") delete missionVariables.ups_trumbles;
}

this.trumbleSound = function trumbleSound()
{
	this.playSound("trumblesqueal.ogg")
}

this.playSound = function (song)
{
	if (!this.mySound) this.mySound = new SoundSource;
	this.mySound.sound = song; // this.playSound("trumblesqueal.ogg");
	this.mySound.play();
}

this.resetMedicines = function (galaxy, success)
{
    var targetSystem = System.infoForSystem(galaxy, this.ups_pplanet);
	if (missionVariables.ups_medicine_government)
	{
		missionVariables.ups_medicine_description = null;
		missionVariables.ups_medicine_inhabitants = null;
		missionVariables.ups_medicine_population = null;
		missionVariables.ups_medicine_government = null;
		missionVariables.ups_medicine_productivity = null;
        if (success)
        {
            // reset descriptions for a successful mission.
            targetSystem.description = null;
            targetSystem.inhabitants = null;
            targetSystem.population = null;
            targetSystem.government = null;
            targetSystem.productivity = null;
        }
		// targetSystem.market = null; // not possible to read correctly in 1.74 ?
	}
	missionVariables.ups_disease = null;
	missionVariables.ups_museumGoods = null;
    this.ups_ptimerstart = null;
}

this._reputation = function ()
{
    return player.parcelReputation;
}

this.setUpsReputation = function ()
{
    if (system.government <= 1) return; // No update in anarchy and feudal!

    var message;
    switch (this._reputation())
    {
        case 7: message = "outstanding"; break;
        case 6: message = "excellent"; break;
        case 5: message = "dedicated"; break;
        case 4: message = "very good"; break;
        case 3: message = "good"; break;
        case 2: message = "above average"; break;
        case 1: message = "average"; break;
        case 0: message = "rookie"; break;
        case -1: message = "incompetent"; break;
        case -2: message = "unreliable"; break;
        case -3: message = "irresponsible"; break;
        default: message = "not trustworthy"; // for all other negative numbers
    }
    mission.setInstructions(expandMissionText("ups_parcel_reputation", {ups_reputation: message}));
}

// Code below is linked to ship scripts
this.sendUpsParcelMessage = function ()
{
	if (worldScripts["ups_docs"].messageAllowed() && this.ship.target && this.ship.target.isPlayer)
		this.ship.commsMessage(expandDescription("[ups_parceltalk]"));
    // no messages within 5 secs to avoid screen clutter.
}

this.sendUpsAmbushMessage = function ()
{
	if (worldScripts["ups_docs"].messageAllowed() && this.ship.target && this.ship.target.isPlayer)
		this.ship.commsMessage(expandDescription("[ups_ambush_talk]"));
    // no messages within 5 secs to avoid screen clutter.
}

this._overlay = function() {
    if (player.ship.hudHidden || player.ship.hudAllowsBigGui) {
        return {name:"upsnews-large.png", height:546};
    } else {
        return {name:"upsnews-small.png", height:546};
    }
}
